model saclay


global
{
	file shape_file_buildings <- file("../includes/osm_polygon.shp");
	file shape_file_roads <- file("../includes/osm_line.shp");
	file shape_file_bounds <- file("../includes/osm_point.shp");
	geometry shape <- envelope(shape_file_bounds);
	list<building> gares;
	list<building> bat_univ;
	int stepsize <- 10;
	int nb_people <- 100;
	int current_hour update: (time / 60 * stepsize) mod 24;
	int min_work_start <- 6;
	int max_work_start <- 8;
	int min_work_end <- 16;
	int max_work_end <- 20;
	float min_speed <- 1.0 # km / # h;
	float max_speed <- 5.0 # km / # h;
	graph the_graph;
	graph the_connected_graph;
	graph not_connected_graph;
	list<geometry> bigcomponent;
	list<road> connected_roads <- [];
	list<road> not_connected_roads <- [];
	road target;
	list<road> busyroads <- [];
	list<string> names <- [""];
	list<int> sumnb <- [0];
	init
	{
		create road from: shape_file_roads with: [type:: string(read("highway"))]
		{
			color <- # blue;
			if type = ""
			{
				write ("kill " + self.name);
				do die;
			}

			if type = "footway"
			{
				color <- # green;
			}

		}

		write ("roads :" + length(road));
		the_graph <- as_edge_graph(road);
		target <- one_of(road where (each.name = "Rue Nicolas Appert"));
		do clean_roads;
		names <- [];
		loop re over: connected_roads
		{
			if not (names contains re.type)
			{
				add re.type to: names;
			}

		}

		create building from: shape_file_buildings with: [type:: string(read("building"))]
		{
			if (type != "yes" and type != "station")
			{ //				write("kill "+self.name);
 do die;
			}

			if name contains "Bâtiment"
			{
				color <- # blue;
			}

			if name != ""
			{
				color <- # blue;
			}

			if (path_between(the_graph, self, target) = nil)
			{
				write ("no path" + self.name);
				color <- # grey;
			}

			gares <- building where (each.color = # red);
			bat_univ <- building where (each.color = # blue);
		}

		write ("buildings :" + length(building));
		write ("bat_univ :" + length(bat_univ));
		create people number: nb_people
		{
			location <- any_location_in(one_of(bat_univ));
			speed <- (min_speed + rnd(max_speed - min_speed)) * stepsize;
			start_work <- min_work_start + rnd(max_work_start - min_work_start);
			end_work <- min_work_end + rnd(max_work_end - min_work_end);
		}

	}

	action clean_roads
	{
		loop r over: road
		{
			if (path_between(the_graph, r, target) != nil)
			{
				add r to: connected_roads;
				write ("add " + r.name);
			} else
			{
				add r to: not_connected_roads;
				write ("NOT  " + r.name);
				r.color <- # red;
			}

		}

		add target to: connected_roads;
		write ("connected :" + length(connected_roads));
		the_connected_graph <- as_edge_graph(connected_roads);
		not_connected_graph <- as_edge_graph(not_connected_roads);
		bool addnew <- true;
		loop ed over: not_connected_roads
		{
			ed.disttograph <- the_connected_graph.vertices min_of ((each as geometry) distance_to ed);
		}

		not_connected_roads <- not_connected_roads sort_by each.disttograph;
		loop nr over: not_connected_roads
		{
			addnew <- false;
			the_connected_graph <- as_edge_graph(connected_roads);
			not_connected_graph <- as_edge_graph([nr]);
			point minnode <- not_connected_graph.vertices[0];
			float mindist <- the_connected_graph.vertices min_of ((each as geometry) distance_to minnode);
			point minnodeinto <- the_connected_graph.vertices with_min_of ((each as geometry) distance_to minnode);
			loop ed over: not_connected_graph.vertices
			{
				if (the_connected_graph.vertices min_of ((each as geometry) distance_to (ed as geometry)) < mindist)
				{
					minnode <- ed;
					mindist <- the_connected_graph.vertices min_of ((each as geometry) distance_to (ed as geometry));
					minnodeinto <- the_connected_graph.vertices with_min_of ((each as geometry) distance_to (ed as geometry));
				}

			}

			list<road> minroads <- not_connected_roads where (each.shape.points contains minnode);
			write ("p1:" + minnode + " pinto " + minnodeinto + "dist" + mindist + "minroads " + minroads);
			the_graph <- add_edge(the_graph, minnode::minnodeinto);
			the_graph <- add_edge(the_graph, minnodeinto::minnode);
			bigcomponent <- the_graph.edges;
			if (length(minroads) > 0)
			{
				loop minroad over: minroads
				{
					if (path_between(the_graph, minroad, target) != nil)
					{
						add minroad to: connected_roads;
						not_connected_roads <- not_connected_roads - minroad;
						write ("add new " + minroad.name);
						minroad.color <- # black;
						addnew <- true;
					} else
					{
						not_connected_roads <- not_connected_roads - minroad;
						write ("not added new " + minroad.name);
						addnew <- true;
					}

				}

			}

		}

		the_connected_graph <- as_edge_graph(connected_roads);
		 		loop ed over:the_graph.vertices
		{
						if (path_between(the_graph,(ed as point),target)=nil)
						{
							write("pb node "+ed);
							point n2<-(the_connected_graph.vertices with_min_of ((each as point) distance_to (ed as point)));
							the_graph<-the_graph add_edge(n2::ed);		
							the_graph<-the_graph add_edge(ed::n2);		
						}						
			
		}
		write ("bc:" + length(connected_components_of(the_graph)) + " det " + length(connected_components_of(the_graph)[0]));
	}

	reflex updategraph
	{
		busyroads <- 10 last (connected_roads sort_by each.nbpeople);
		int total <- sum(connected_roads collect each.nbpeople) + 1;
		if ((time mod 10) = 0)
		{
			write ("--- new total:" + total);
			loop ro over: busyroads
			{
				write ("road " + ro.name + " nb: " + (ro.nbpeople * 100 / total) + "%");
			}

		}

		sumnb <- [];
		loop ty over: names
		{
			list<road> ro <- (connected_roads where (each.type contains ty));
			if (length(ro) = 0)
			{
				add 0 to: sumnb;
			} else
			{
				add sum(ro collect each.nbpeople) to: sumnb;
			}

		}

	}

}

species building
{
	string type;
	rgb color <- # gray;
	aspect base
	{
		draw shape color: color;
	}

}

species road
{
	float disttograph <- 0;
	string type;
	rgb color <- # black;
	int nbpeople <- 0;
	aspect base
	{ //		if (color!=#red) 
 //		{
 draw shape color: # black;
		if (busyroads contains self)
		{
			draw shape color: # red;
		} //		}
 }

}

species people skills: [moving] control: fsm
{
	rgb color <- # yellow;
	building gare_place <- nil;
	building working_place <- nil;
	int start_work;
	int end_work;
	point the_target <- nil;
	path currentpath;
	aspect base
	{
		draw circle(10) color: color;
	}

	state waiting initial: true
	{
		transition to: moving when: (current_hour = start_work)
		{
			building bt <- one_of(bat_univ where (each.name contains "Lovelace"));
			the_target <- any_location_in(bt);
			location <- the_target;
			building bt <- one_of(bat_univ where (each.name contains "CESFO du Plateau"));
			the_target <- any_location_in(bt);
			currentpath <- path_between(the_graph, location, the_target);
			if (currentpath = nil)
			{
				write "no path to" + bt.name;
				location <- the_target;
			} else
			{
				loop ro over: currentpath.edges
				{
					if (species(ro) = road)
					{
						(ro as road).nbpeople <- (ro as road).nbpeople + 1;
					}

				}

			}

		}

	}

	state moving
	{
		do goto target: the_target on: the_graph;
		transition to: waiting when: the_target.location = location
		{
			the_target <- nil;
		}

	}

}

experiment saclay type: gui
{
	parameter "Shapefile for the buildings:" var: shape_file_buildings category: "GIS";
	parameter "Shapefile for the roads:" var: shape_file_roads category: "GIS";
	parameter "Shapefile for the bounds:" var: shape_file_bounds category: "GIS";
	parameter "Number of people agents" var: nb_people category: "People";
	output
	{
		display city_display type: opengl
		{
			species building aspect: base;
			species road aspect: base;
			species people aspect: base;
		}

		display chart_display refresh_every: 10
		{
			chart "Busiest" type: histogram size: { 1, 0.5 } position: { 0, 0 }
			{
				datalist value: (busyroads collect each.nbpeople) style: bar;
			}

			chart "By Category" type: pie size: { 1, 0.5 } position: { 0, 0.5 }
			{
				datalist legend: names value: sumnb;
			}

		}

	}

}